import 'dart:async';
import 'dart:convert';
import 'package:feed_plugin/models/hashtag.dart';
import 'package:feed_plugin/models/hashtagPostsModel.dart';
import 'package:feed_plugin/models/log.dart';
import 'package:feed_plugin/models/media.dart';
import 'package:feed_plugin/models/poll_option_model.dart';
import 'package:feed_plugin/models/poll_question_model.dart';
import 'package:feed_plugin/models/poll_vote_model.dart';
import 'package:feed_plugin/models/post_comment_model.dart';
import 'package:feed_plugin/models/post_model.dart';
import 'package:feed_plugin/models/replyCommentModel.dart';
import 'package:feed_plugin/models/report_model.dart';
import 'package:feed_plugin/models/report_sub_type_model.dart';
import 'package:feed_plugin/models/report_type_model.dart';
import 'package:feed_plugin/models/story/storyModel.dart';
import 'package:feed_plugin/models/user_model.dart';
import 'package:feed_plugin/service/http_service.dart';
import 'package:feed_plugin/utilities/constants.dart';
import 'package:feed_plugin/utilities/extensions.dart';
import 'package:flutter/cupertino.dart';
import 'package:network_info_plus/network_info_plus.dart';
import 'package:package_info/package_info.dart';
import 'package:uuid/uuid.dart';
import 'package:feed_plugin/utilities/global_vars.dart' as global_vars;

class FeedRepository {
  final HttpService _httpService;

  FeedRepository(this._httpService) : assert(_httpService != null);

  PackageInfo _packageInfo;
  UserModel _user;

  PackageInfo get packageInfo => _packageInfo;

  UserModel get currentUser => _user;

  List<String> get enabledLogsList => global_vars.enabledLogsList;

  void setPackageInfo() async {
    final pi = await PackageInfo.fromPlatform();
    _packageInfo = pi;
  }

  void fetchEnableLogs() {
    _httpService.fetchEnableLogs().then((data) {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        final body = json.decode(data.body);
        List list1 = body['hydra:member'];
        var list = list1.map((e) => e['logName'].toString()).toList();
        global_vars.enabledLogsList = list;
      } else {
        global_vars.enabledLogsList = [];
      }
    });
  }

  Future<List<PostModel>> loadPosts() {
    return _httpService.fetchAllPosts().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> posts = json.decode(data.body);
        List<PostModel> postsList =
            posts.map((i) => PostModel.fromJson(i)).toList();
        return postsList;
      } else {
        return List<PostModel>.from([]);
      }
    });
  }

  Future<PostModel> addPost(
      {user, title, description, type, attachedUrl, pollQuestionId}) {
    Log(
      logName: 'AddPost',
      className: 'Post',
      methodName: 'addPost',
      type: 'INFO',
      text:
          '{event: Add Post, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, user id: ${user.id}, title: $title, description: $description, type: $type, attachedUrl: $attachedUrl, poll question id: $pollQuestionId',
    );
    return _httpService
        .addPost(
            user: user,
            description: description,
            type: type,
            attachedUrl: attachedUrl,
            title: title,
            pollQuestionId: pollQuestionId)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        List<dynamic> res = json.decode(data.body);
        List<PostModel> postData =
            res.map((i) => PostModel.fromJson(i)).toList();
        if (postData[0] != null) return postData[0];

        return null;
      }
      return null;
    });
  }

  Future<List<ReportTypeModel>> fetchAllReports() {
    return _httpService.fetchAllReports().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> reports = json.decode(data.body);
        List<ReportTypeModel> reportsTypeList =
            reports.map((i) => ReportTypeModel.fromJson(i)).toList();
        return reportsTypeList;
      } else {
        return List<ReportTypeModel>.from([]);
      }
    });
  }

  Future<List<ReportSubTypeModel>> fetchSubReport(reportTypeId) {
    return _httpService.fetchSubReport(reportTypeId).then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> reports = json.decode(data.body);
        List<ReportSubTypeModel> reportsSubTypeList =
            reports.map((i) => ReportSubTypeModel.fromJson(i)).toList();
        return reportsSubTypeList;
      } else {
        return List<ReportSubTypeModel>.from([]);
      }
    });
  }

  Future<UserModel> fetchUserData(userId) {
    return _httpService.fetchUser(userId).then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> res = json.decode(data.body);
        dynamic userData = res[0];
        UserModel user = UserModel.fromJson(userData);
        return user;
      } else {
        return null;
      }
    });
  }

  Future<ReportModel> addReport(typeId, userId, postId) {
    Log(
      logName: 'AddReport',
      className: 'Report',
      methodName: 'addReport',
      type: 'INFO',
      text:
          '{event: Add Report, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, user id: $userId, post id: $postId, type id: $typeId',
    );
    return _httpService
        .addReport(typeId: typeId, userId: userId, postId: postId)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);
        ReportModel reportData = ReportModel.fromJson(res[0]);
        return reportData;
      }
      return null;
    });
  }

  Future<bool> updatePostReport(postId) {
    return _httpService.updatePostReport(postId: postId).then((data) async {
      if (data.statusCode >= 200) {
        return true;
      }
      return false;
    });
  }

  Future<UserModel> authenticatePartner(
      {firstName, lastName, about, profileImg, token, socialMedia, partnerId}) {
    Log(
      logName: 'AuthenticatePartner',
      className: 'User',
      methodName: 'authenticatePartner',
      type: 'INFO',
      text:
          '{event: Authenticate Partner, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, prtaner info: first name- $firstName, profile image- $profileImg, social media- ${socialMedia.toString()}',
    );
    return _httpService
        .authenticateUser(
            firstName: firstName,
            lastName: lastName,
            about: about,
            profileImg: profileImg,
            token: token,
            partnerId: partnerId)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);
        UserModel userData;
        if (socialMedia != null) {
          userData = await updateUserMedia(
              socialMedia: socialMedia, userId: res['id']);
        } else {
          userData = UserModel.fromJson(res);
        }
        return userData;
      }
      return null;
    });
  }

  Future<UserModel> updateUserMedia({socialMedia, userId}) async {
    Log(
      logName: 'UpdateUserMedia',
      className: 'User',
      methodName: 'updateUserMedia',
      type: 'INFO',
      text:
          '{event: Update User Media, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, user id: $userId, social media: ${socialMedia.toString()}',
    );
    UserModel userModel;
    return await _httpService
        .updateUserMedia(socialMedia: socialMedia, userId: userId)
        .then((user) {
      if (user.statusCode >= 200 && user.statusCode < 300) {
        var res = json.decode(user.body);
        userModel = UserModel.fromJson(res);
        return userModel;
      }
      return null;
    });
  }

  Future<UserModel> authenticateUser(
      {firstName, lastName, about, profileImg, token, socialMedia, partnerId}) {
    Log(
      logName: 'AuthenticateUser',
      className: 'User',
      methodName: 'authenticateUser',
      type: 'INFO',
      text:
          '{event: Authenticate User, first name: $firstName, last name: $lastName, about: $about, profile image: $profileImg, partner id: $partnerId, ',
    );
    return _httpService
        .authenticateUser(
            firstName: firstName,
            lastName: lastName,
            about: about,
            profileImg: profileImg,
            token: token,
            partnerId: partnerId)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);
        UserModel userData = UserModel.fromJson(res);
        _user = userData;
        return userData;
      }
      return null;
    });
  }

  Future<bool> removeReport(reportId) {
    Log(
      logName: 'RemoveReport',
      className: 'Report',
      methodName: 'removeReport',
      type: 'INFO',
      text:
          '{event: Remove Report, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, report id: $reportId',
    );
    return _httpService.removeReport(reportId: reportId).then((data) async {
      if (data.statusCode == 200) {
        return true;
      }
      return false;
    });
  }

  Future<bool> removeComment(commentId) {
    Log(
      logName: 'RemoveComment',
      className: 'Comment',
      methodName: 'removeComment',
      type: 'INFO',
      text:
          '{event: Remove Comment, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, comment id: $commentId',
    );
    return _httpService.removeComment(commentId: commentId).then((data) async {
      if (data.statusCode == 200) {
        return true;
      }
      return false;
    });
  }

  Future<List<ReportModel>> fetchPostReports(postId) {
    Log(
      logName: 'FetchPostReports',
      className: 'Report',
      methodName: 'fetchPostReports',
      type: 'INFO',
      text:
          '{event: Fetch Post Reports, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, post id: $postId',
    );
    return _httpService.fetchReportByPostId(postId).then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> reports = json.decode(data.body);
        List<ReportModel> reportsList =
            reports.map((i) => ReportModel.fromJson(i)).toList();
        return reportsList;
      } else {
        return List<ReportModel>.from([]);
      }
    });
  }

  Future<List<PostCommentModel>> fetchPostComments(postId) async {
    Log(
      logName: 'FetchPostComments',
      className: 'Comment',
      methodName: 'fetchPostComments',
      type: 'INFO',
      text:
          '{event: Fetch Post Comments, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, post id: $postId',
    );
    return _httpService.fetchCommentsByPostId(postId).then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> comments = json.decode(data.body);
        List<PostCommentModel> commentsList =
            comments.map((i) => PostCommentModel.fromJson(i)).toList();
        return commentsList;
      } else {
        return List<PostCommentModel>.from([]);
      }
    });
  }

  Future<List<PostModel>> fetchPostsByUserId(id) async {
    Log(
      logName: 'FetchPostsByUserId',
      className: 'Post',
      methodName: 'fetchPostsByUserId',
      type: 'INFO',
      text:
          '{event: Fetch Posts By User Id, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, user id: $id',
    );
    return _httpService.fetchPostsByUserId(id).then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> posts = json.decode(data.body);
        List<PostModel> postsList =
            posts.map((i) => PostModel.fromJson(i)).toList();
        return postsList;
      } else {
        return List<PostModel>.from([]);
      }
    });
  }

  Future<List<ReportModel>> fetchFavoritePosts(currentUserId, saveId) async {
    Log(
      logName: 'FetchFavoritePosts',
      className: 'Post',
      methodName: 'fetchFavoritePosts',
      type: 'INFO',
      text:
          '{event: Fetch Favorite Posts, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, user id: $currentUserId save id: $saveId',
    );
    return _httpService
        .fetchFavoritePosts(currentUserId, saveId)
        .then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> posts = json.decode(data.body);
        List<ReportModel> postsList =
            posts.map((i) => ReportModel.fromJson(i)).toList();
        return postsList;
      } else {
        return List<ReportModel>.from([]);
      }
    });
  }

  Future<List<ReportSubTypeModel>> fetchAllSubType() async {
    return _httpService.fetchAllSubType().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> reports = json.decode(data.body);
        List<ReportSubTypeModel> reportsSubTypeList =
            reports.map((i) => ReportSubTypeModel.fromJson(i)).toList();
        return reportsSubTypeList;
      } else {
        return List<ReportSubTypeModel>.from([]);
      }
    });
  }

  Future<bool> removePost(postId) {
    Log(
      logName: 'RemovePost',
      className: 'Post',
      methodName: 'removePost',
      type: 'INFO',
      text:
          '{event: Remove Post, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, post id: $postId',
    );
    return _httpService.removePost(postId: postId).then((data) async {
      if (data.statusCode == 200) {
        return true;
      }
      return false;
    });
  }

  Future<StoryModel> removePage(int id, int numPage) {
    Log(
      logName: 'RemoveStoryPage',
      className: 'Story',
      methodName: 'removePage',
      type: 'INFO',
      text:
          '{event: Remove Story Page, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, story id: $id, page: $numPage',
    );
    return _httpService
        .removePage(storyId: id, numPage: numPage)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        if (data.body != '' && data.body != null) {
          var res = json.decode(data.body);
          if (res != null && res.length > 0 && res[0] != null) {
            StoryModel storyData = StoryModel.fromJson(res[0]);
            return storyData;
          }
        }
      }
      return null;
    });
  }

  Future<List<HashtagModel>> getAllHashtags() {
    return _httpService.getAllHashtags().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> tags = json.decode(data.body);
        List<HashtagModel> tagsList =
            tags.map((i) => HashtagModel.fromJson(i)).toList();
        return tagsList;
      } else {
        return List<HashtagModel>.from([]);
      }
    });
  }

  Future<HashtagPostsModel> addhashtagPosts(tagId, postId) {
    Log(
      logName: 'AddhashtagPost',
      className: 'Hashtag',
      methodName: 'addhashtagPosts',
      type: 'INFO',
      text:
          '{event: Add hashtag Post, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, post id: $postId, tag id: $tagId',
    );
    return _httpService
        .addhashtagPosts(tagId: tagId, postId: postId)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);
        HashtagPostsModel tagPostData = HashtagPostsModel.fromJson(res[0]);
        return tagPostData;
      }
      return null;
    });
  }

  Future<HashtagModel> addNewhashtag(tagName) {
    Log(
      logName: 'AddNewhashtag',
      className: 'Hashtag',
      methodName: 'addNewhashtag',
      type: 'INFO',
      text:
          '{event: Add New hashtag, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, tag name: $tagName',
    );
    return _httpService.addNewhashtag(tagName: tagName).then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);
        HashtagModel tagData = HashtagModel.fromJson(res[0]);
        return tagData;
      }
      return null;
    });
  }

  Future<List<HashtagPostsModel>> getHashtagPostByTagName(String tagName) {
    Log(
      logName: 'GetHashtagPostByTagName',
      className: 'Hashtag',
      methodName: 'getHashtagPostByTagName',
      type: 'INFO',
      text:
          '{event: Get Hashtag Post By Tag Name, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, tag name: $tagName',
    );
    return _httpService
        .getHashtagByTagName(tagName.substring(1))
        .then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> tags = json.decode(data.body);
        List<HashtagPostsModel> tagsList =
            tags.map((i) => HashtagPostsModel.fromJson(i)).toList();
        return tagsList;
      } else {
        return null;
      }
    });
  }

  Future<List<UserModel>> getUsersRandom(skip) {
    return _httpService.getUsersRandom(skip).then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> users = json.decode(data.body);
        List<UserModel> usersList =
            users.map((i) => UserModel.fromJson(i)).toList();
        return usersList;
      } else {
        return null;
      }
    });
  }

  Future<ReplyCommentModel> addReplyComment(postComments, user, comment) {
    Log(
      logName: 'AddReplyComment',
      className: 'Comment',
      methodName: 'addReplyComment',
      type: 'INFO',
      text:
          '{event: Add Reply Comment, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, post id: $postComments, comment: $comment',
    );
    return _httpService
        .addReplyComment(
            postComments: postComments, user: user, comment: comment)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        List<dynamic> res = json.decode(data.body);
        ReplyCommentModel replyComment = ReplyCommentModel.fromJson(res[0]);
        if (replyComment != null) return replyComment;

        return null;
      }
      return null;
    });
  }

  Future<List<ReplyCommentModel>> getReplayCommentsList() {
    return _httpService.getReplayCommentsList().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> users = json.decode(data.body);
        List<ReplyCommentModel> replyList =
            users.map((i) => ReplyCommentModel.fromJson(i)).toList();
        return replyList;
      } else {
        return null;
      }
    });
  }

  Future<bool> removeReplyComment(replyId) {
    Log(
      logName: 'RemoveReplyComment',
      className: 'Comment',
      methodName: 'removeReplyComment',
      type: 'INFO',
      text:
          '{event: Remove Reply Comment, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, reply id: $replyId',
    );
    return _httpService.removeReplyComment(replyId: replyId).then((data) async {
      if (data.statusCode == 200) {
        return true;
      }
      return false;
    });
  }

  Future<PollQuestionModel> createPollQuestion(question) {
    Log(
      logName: 'CreatePollQuestion',
      className: 'Poll',
      methodName: 'createPollQuestion',
      type: 'INFO',
      text:
          '{event: Create Poll Question, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, uestion: $question',
    );
    return _httpService
        .createPollQuestion(question: question)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);

        PollQuestionModel pollQuestionData = PollQuestionModel.fromJson(res[0]);
        return pollQuestionData;
      }
      return null;
    });
  }

  Future<PollOptionModel> createPollOption(pollQuestion, option) {
    Log(
      logName: 'CreatePollOption',
      className: 'Poll',
      methodName: 'createPollOption',
      type: 'INFO',
      text:
          '{event: Create Poll Option, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, poll question: $pollQuestion, option: $option',
    );
    return _httpService
        .createPollOption(pollQuestion: pollQuestion, option: option)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);

        PollOptionModel pollOptionData = PollOptionModel.fromJson(res[0]);
        return pollOptionData;
      }
      return null;
    });
  }

  Future<List<PollOptionModel>> getAllOptions() {
    return _httpService.getAllOptions().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> tags = json.decode(data.body);
        List<PollOptionModel> pollOptionsList =
            tags.map((i) => PollOptionModel.fromJson(i)).toList();
        return pollOptionsList;
      } else {
        return List<PollOptionModel>.from([]);
      }
    });
  }

  Future<List<PollOptionModel>> getOptionsByQuestionId(questionId) {
    Log(
      logName: 'GetOptionsByQuestionId',
      className: 'Poll',
      methodName: 'getOptionsByQuestionId',
      type: 'INFO',
      text:
          '{event: Get Options By Question Id, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, question id: $questionId',
    );
    return _httpService.getOptionsByQuestionId(questionId).then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> options = json.decode(data.body);
        List<PollOptionModel> optionsList =
            options.map((i) => PollOptionModel.fromJson(i)).toList();
        return optionsList;
      } else {
        return List<PollOptionModel>.from([]);
      }
    });
  }

  Future<List<PollVoteModel>> getVotesByOptionId(optionId) {
    Log(
      logName: 'GetVotesByOptionId',
      className: 'Poll',
      methodName: 'getVotesByOptionId',
      type: 'INFO',
      text:
          '{event: Get Votes By Option Id, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, option id: $optionId',
    );
    return _httpService.getVotesByOptionId(optionId).then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> votes = json.decode(data.body);
        List<PollVoteModel> votesList =
            votes.map((i) => PollVoteModel.fromJson(i)).toList();
        return votesList;
      } else {
        return List<PollVoteModel>.from([]);
      }
    });
  }

  Future<PollVoteModel> createPollVote(pollOption, user) {
    Log(
      logName: 'CreatePollVote',
      className: 'Poll',
      methodName: 'createPollVote',
      type: 'INFO',
      text:
          '{event: Create Poll Vote, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, poll vote option: $pollOption',
    );
    return _httpService
        .createPollVote(pollOption: pollOption, user: user)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);

        PollVoteModel pollVoteData = PollVoteModel.fromJson(res[0]);
        return pollVoteData;
      }
      return null;
    });
  }

  Future<bool> removePollVote(pollVoteId) {
    Log(
      logName: 'RemovePollVote',
      className: 'Poll',
      methodName: 'removePollVote',
      type: 'INFO',
      text:
          '{event: Remove Poll Vote, user: ${currentUser?.userPayloadId}, user is ${currentUser?.firstName} ${currentUser?.lastName}, poll vote id: id- $pollVoteId',
    );
    return _httpService
        .removePollVote(pollVoteId: pollVoteId)
        .then((data) async {
      if (data.statusCode == 200) {
        return true;
      }
      return false;
    });
  }

  Future<List<PollVoteModel>> getAllVotes() {
    return _httpService.getAllVotes().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> tags = json.decode(data.body);
        List<PollVoteModel> pollVotesList =
            tags.map((i) => PollVoteModel.fromJson(i)).toList();
        return pollVotesList;
      } else {
        return List<PollVoteModel>.from([]);
      }
    });
  }

  Future<List<StoryModel>> getAllStories() {
    return _httpService.getAllStories().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> story = json.decode(data.body);
        List<StoryModel> storiesList =
            story.map((i) => StoryModel.fromJson(i)).toList();
        return storiesList;
      } else {
        return List<StoryModel>.from([]);
      }
    });
  }

  Future<List<StoryModel>> getStoriesToday() {
    return _httpService.getAllStories().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> story = json.decode(data.body);
        List<StoryModel> storiesList =
            story.map((i) => StoryModel.fromJson(i)).toList();
        final date = DateTime.now();
        List<StoryModel> storyToday = storiesList
            .where((s) => date.difference(s.createdAt).inDays < 1)
            .toList();
        return storyToday;
      } else {
        return List<StoryModel>.from([]);
      }
    });
  }

  Future<StoryModel> createStory({text, user, attachedUrl, type}) {
    Log(
      logName: 'CreateStory',
      className: 'Story',
      methodName: 'createStory',
      type: 'INFO',
      text:
          '{event: Create Story, user: ${user?.userPayloadId.toString()}, user is ${user?.firstName.toString()} ${user?.lastName.toString()}, story info: attachedUrl- $attachedUrl, text- $text, type- $type',
    );
    return _httpService
        .createStory(
            text: text, attachedUrl: attachedUrl, user: user, type: type)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);
        StoryModel storyData = StoryModel.fromJson(res[0]);
        return storyData;
      }
      return null;
    });
  }

  Future<bool> updateStory({id, attachedUrl, text, type}) {
    Log(
      logName: 'UpdateStory',
      className: 'Story',
      methodName: 'updateStory',
      type: 'INFO',
      text:
          '{event: Update Story, user: ${currentUser?.userPayloadId}}, user is ${currentUser?.firstName} ${currentUser?.lastName}, story info: id- $id, attachedUrl- $attachedUrl, text- $text, type- $type',
    );
    return _httpService
        .updateStory(
            storyId: id, attachedUrl: attachedUrl, text: text, type: type)
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);
        return true;
      }
      return null;
    });
  }

  void createRemoteLog(Map<String, dynamic> data) {
    _httpService.createLog(data);
  }

  // ignore: non_constant_identifier_names
  void Log({
    @required String className,
    @required String methodName,
    @required String text,
    @required String type,
    @required String logName,
  }) async {
    if (enabledLogsList != null &&
        enabledLogsList.contains('$BASE_LOG_NAME$logName')) {
      try {
        if (_packageInfo == null) setPackageInfo();
        final info = NetworkInfo();
        var wifiIP = await info.getWifiIP();
        var log = LogModel(
          id: Uuid().v4(),
          createdAt: DateTime.now(),
          fromDevice: 'mobile',
          message: 'logName: $BASE_LOG_NAME$logName, methodName: $methodName, ' +
              text +
              ', createdAt: ${DateTime.now().dateToString()}, App version: ${packageInfo.version} + ${packageInfo.buildNumber}',
          ip: wifiIP,
          type: type,
          title: className,
        );
        createRemoteLog(log.toJson());
      } catch (e) {
        final info = NetworkInfo();
        var wifiIP = await info.getWifiIP();
        var log = LogModel(
          id: Uuid().v4(),
          createdAt: DateTime.now(),
          fromDevice: 'mobile',
          message:
              'logName: $BASE_LOG_NAME$logName, methodName: Log, error: $e, createdAt: ${DateTime.now().dateToString()}, App version: ${packageInfo.version} + ${packageInfo.buildNumber}',
          ip: wifiIP,
          type: 'ERROR',
          title: className,
        );
        createRemoteLog(log.toJson());
        var log2 = LogModel(
          id: Uuid().v4(),
          createdAt: DateTime.now(),
          fromDevice: 'mobile',
          message:
              '[catch] logName: $BASE_LOG_NAME$logName, methodName: $methodName, ' +
                  text +
                  ', createdAt: ${DateTime.now().dateToString()}, App version: ${packageInfo.version} + ${packageInfo.buildNumber}',
          ip: wifiIP,
          type: type,
          title: className,
        );
        createRemoteLog(log2.toJson());
      }
    }
  }
}
